;;; webapp project

(defpackage #:webapp
  (:use #:common-lisp #:net.aserve #:net.html.generator))

(in-package :webapp)

(defparameter *webapp-home*
  (let ((load-name (or *compile-file-truename* *load-truename*)))
    (namestring
     (make-pathname :host (pathname-host load-name)
                    :device (pathname-device load-name)
                    :directory (pathname-directory load-name)))))  

(webaction-project 
 "webapp"
 :project-prefix "/webapp/"
 :destination *webapp-home*
 :index "home"
 :map
 '(;; respond to a GET
   ("login" "login.clp")
   ("home" check-login "home.clp")
   
   ;; respond to a POST
   ("do-login" do-login "home" (:redirect t))
   ))


;;; CHECK-LOGIN
;;;   continue to requested page if user name is known, 
;;;   otherwise save requested page and go to login

(defun check-login (req resp)
  (declare (ignore resp))
  (setf (websession-variable (websession-from-req req) "error") nil)
  (cond ((websession-variable (websession-from-req req) "username")
         :continue)
        (t
         (setf (request-variable-value req "after-login") 
           (net.uri:uri-path (request-uri req)))
         "login")))

;;; DO-LOGIN
;;;   called when user submits the login.clp form
;;;   If valid name and password are entered, save the name
;;;   and go to the requested page, if any, or just continue.
;;;   If not valid, go back to the login page.

(defun do-login (req resp)
  (declare (ignore resp))
  (let ((name (request-query-value "username" req))
        (password (request-query-value "password" req))
        (after (request-query-value "after-login" req)))
    (cond ((password-valid-p name password)
           (setf (websession-variable (websession-from-req req) "username") name)
           (if (empty-arg-p after) :continue after))
          (t
           (setf (websession-variable (websession-from-req req) "error")
             "name and/or password are invalid")
           "login"))))

;;; replace with something real--this just requires a password equal
;;; to the user name reversed
(defun password-valid-p (name password)
  (and (not (empty-p name))
       (equal password (reverse name))))